@@ -217,7 +217,7 @@ class VCRConnection(object):
             response = self.cassette.play_response(self._vcr_request)
             return VCRHTTPResponse(response)
         else:
-            if self.cassette.write_protected and self.cassette._filter_request(self._vcr_request):
+            if self.cassette.write_protected and self.cassette.filter_request(self._vcr_request):
                 raise CannotOverwriteExistingCassetteException(
                     "Can't overwrite existing cassette (%r) in "
                     "your current record mode (%r)."
